@@ -10,25 +10,22 @@ use std::{
 
 fn main() -> io::Result<()> {
     println!("cargo:rerun-if-env-changed=VCPKG_ROOT");
-    let vcpkg_root = env::var("VCPKG_ROOT").map_or_else(
-        |_| {
-            // Try to find %LOCALAPPDATA%\vcpkg\vcpkg.path.txt if %VCPKG_ROOT% was not set.
-            println!("cargo:rerun-if-env-changed=LOCALAPPDATA");
-            let mut vcpkg_app_data = PathBuf::from(env!("LOCALAPPDATA"));
-            vcpkg_app_data.push("vcpkg");
-            vcpkg_app_data.push("vcpkg.path.txt");
-            println!("cargo:rerun-if-changed={}", vcpkg_app_data.display());
-            let mut vcpkg_path_txt = File::open(&vcpkg_app_data)
-                .expect(format!("Failed to open: {}", vcpkg_app_data.display()).as_str());
-            let mut buf = Vec::new();
-            vcpkg_path_txt
-                .read_to_end(&mut buf)
-                .expect(format!("Failed to read: {}", vcpkg_app_data.display()).as_str());
-            String::from_utf8(buf)
-                .expect(format!("Failed to decode: {}", vcpkg_app_data.display()).as_str())
-        },
-        |value| value,
-    );
+    let vcpkg_root = env::var("VCPKG_ROOT").unwrap_or_else(|_| {
+        // Try to find %LOCALAPPDATA%\vcpkg\vcpkg.path.txt if %VCPKG_ROOT% was not set.
+        println!("cargo:rerun-if-env-changed=LOCALAPPDATA");
+        let mut vcpkg_app_data = PathBuf::from(env!("LOCALAPPDATA"));
+        vcpkg_app_data.push("vcpkg");
+        vcpkg_app_data.push("vcpkg.path.txt");
+        println!("cargo:rerun-if-changed={}", vcpkg_app_data.display());
+        let mut vcpkg_path_txt = File::open(&vcpkg_app_data)
+            .unwrap_or_else(|_| panic!("Failed to open: {}", vcpkg_app_data.display()));
+        let mut buf = Vec::new();
+        vcpkg_path_txt
+            .read_to_end(&mut buf)
+            .unwrap_or_else(|_| panic!("Failed to read: {}", vcpkg_app_data.display()));
+        String::from_utf8(buf)
+            .unwrap_or_else(|_| panic!("Failed to decode: {}", vcpkg_app_data.display()))
+    });
 
     let platform = if cfg!(target_pointer_width = "64") {
         "x64-windows"
@@ -51,6 +48,7 @@ fn main() -> io::Result<()> {
         .define("BUILD_SHARED_LIBS", if vcpkg_static { "OFF" } else { "ON" })
         .define("BUILD_TESTING", "OFF")
         .cxxflag("/EHsc")
+        .generator("Ninja")
         .profile("RelWithDebInfo")
         .build();
 
@@ -73,7 +71,7 @@ fn main() -> io::Result<()> {
         let cpp_libs = [
             "gqlmapi",
             "gqlmapiCommon",
-            "mapischema",
+            "mapi_schema",
             "mapistub",
             "graphqlservice",
             "graphqlintrospection",
@@ -108,29 +106,10 @@ fn main() -> io::Result<()> {
         .file("src/Bindings.cpp")
         .include(gqlmapi_include)
         .include(vcpkg_include)
-        .flag_if_supported("/std:c++17")
+        .flag_if_supported("/std:c++20")
         .flag_if_supported("/EHsc")
         .static_crt(vcpkg_static)
         .compile("gqlmapi-rs");
 
-    windows::build! {
-        Windows::Win32::{
-            Foundation::{
-                HWND,
-                LPARAM,
-                WPARAM,
-            },
-            System::Threading::GetCurrentThreadId,
-            UI::WindowsAndMessaging::{
-                MSG,
-                WM_APP,
-                DispatchMessageA,
-                GetMessageA,
-                PostThreadMessageA,
-                TranslateMessage,
-            },
-        }
-    };
-
     Ok(())
 }
